#!/bin/bash
# © Copyright [2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

# Log files
TMP_LOG_FILE="/tmp/cis_upgrade.log"
DATE_NOW=$(date +%s)
LOG_FILE="/var/opt/novell/log/cis/cis_upgrade_$DATE_NOW.log"

## Patch version
SP1_PATCH_VERSION_NAME="sp1update9"
SP2_OLD_PATCH_VERSION_NAME="sp2update2"
SP2_U4_PATCH_VERSION_NAME="sp2update4"
SP3_0="sp3"

usage(){
cat << EOF
CIS upgrade utility

Usage: cis_upgrade.sh <OPTIONS>

GENERAL OPTIONS (OPTIONS):
  -v, --version         Displays the program version information.
  -h, --help            Displays the help information.

This utility will upgrade CIS to CIS SP 3.0
If CIS files are already stored on media, they will continue to be on media after the upgrade.
If CIS configuration and log files were never moved to media, this utility will treat "/" as media.
EOF
}

version(){
  # Get Server version.
  oes_version=`grep "VERSION" /etc/novell-release | cut -d'=' -f2- | xargs`
  echo -e "CIS upgrade utility version $oes_version"
}

prepareSymLinks(){
  echo -e "Preparing configuration and log path symbolic links." | tee -a "$TMP_LOG_FILE"
  etc_path="/etc/opt/novell/cis_local"
  log_path="/var/opt/novell/log/cis_local"
  if { [ "$1" != "" ] && [ "$2" != "" ]; };then
    etc_path="$1"
    log_path="$2"
  fi
  if [ ! -d /etc/opt/novell/cis_local ] ; then
    if [ ! -L /etc/opt/novell/cis ] ; then
      /usr/bin/mv  /etc/opt/novell/cis /etc/opt/novell/cis_local
      ln -fsn "$etc_path" /etc/opt/novell/cis
    fi
  fi
  if [ ! -d /var/opt/novell/log/cis_local ] ; then
    if [ ! -L /var/opt/novell/log/cis ] ; then
      /usr/bin/mv  /var/opt/novell/log/cis /var/opt/novell/log/cis_local
      ln -fsn "$log_path" /var/opt/novell/log/cis
    fi
  fi
  echo -e "Completed configuration and log path symbolic links." | tee -a "$TMP_LOG_FILE"
}

printandExit(){
	if [[ $1 -ne 0 ]]; then
    echo -e "CIS service upgradation failed. Check /var/opt/novell/log/cis/cis_upgrade*.log"
		echo -e "CIS service upgradation failed." | tee -a "$TMP_LOG_FILE"
    mv "$TMP_LOG_FILE" "$LOG_FILE"
		exit 1
	else
    echo -e "CIS service upgrade completed successfully. Check /var/opt/novell/log/cis/cis_upgrade*.log " 
		echo -e "CIS service upgrade completed successfully." | tee -a "$TMP_LOG_FILE"
    mv "$TMP_LOG_FILE" "$LOG_FILE"
		exit 0
	fi
}

updateConfigStatus(){
  prefix_path=$1  
  blank=""
  configStatus="/etc/opt/novell/cis/configurationStatus.json"
  INLINE_CODE="import json,sys;obj=json.load(open('$configStatus'));obj['prefixPath']=\"$prefix_path\";obj['adminPassword']=\"$blank\";obj['dbPass']=\"$blank\";obj['patch']=\"$SP3_0\"; print json.dumps(obj)"
  
  python -c "$INLINE_CODE" > "$configStatus".tmp
  mv "$configStatus".tmp "$configStatus"
}


clearPasswdInConfigStatus(){
  configStatus="/etc/opt/novell/cis/configurationStatus.json"
  blank=""
  INLINE_CODE="import json,sys;obj=json.load(sys.stdin);obj['adminPassword']=\"$blank\";obj['dbPass']=\"$blank\"; print json.dumps(obj)"
  cat "$configStatus" | python -c "$INLINE_CODE" > "$configStatus".tmp
  mv "$configStatus".tmp "$configStatus"
}

isStringPresent() {
  line=$1
  file=$2
  notfound=`grep "$line" "$file"`
  if [ -z "$notfound" ]; then
    return 0
  else
    return 1
  fi
}

stopServices(){
    # Stopping all CIS core services, If running
    declare -a cis_service_names=(
        "gateway" "mgmt"
        "collector" "policy"
        "repcollector" "aggregator"
        "metadata" "auth"
        "data" "repaggregator"           
    )

    for name in "${cis_service_names[@]}"
    do
      echo -e "Stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
      systemctl stop oes-cis-"$name".service

      if [ $? -ne 0 ]; then
        echo -e "Unable to stop oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
        printandExit 1
      fi

    done
}

writeDockerNwToConf(){
  conf="/etc/opt/novell/cis/config"
  subnet=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
  if [ $? -ne 0 ]; then
    echo -e "Unable to get cisnet subnet." | tee -a "$TMP_LOG_FILE"    
    printandExit 1;
  fi

  gateway=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Gateway}}" cisnet)
  if [ $? -ne 0 ]; then
    echo -e "Unable to get cisnet gateway." | tee -a "$TMP_LOG_FILE"    
    printandExit 1;
  fi

  sub=${subnet/\/*/}  

  IFS="."
  set -- $sub
  range=$((0x$(printf "%02x%02x%02x%02x\n" $1 $2 $3 $4)))
  set -- $gateway
  g=$4

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_DATA/ c\IP_DATA="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_METADATA/ c\IP_METADATA="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_POLICY/ c\IP_POLICY="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_AUTH/ c\IP_AUTH="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_MGMT/ c\IP_MGMT="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_COLLECTOR/ c\IP_COLLECTOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_AGGREGATOR/ c\IP_AGGREGATOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_REPCOLLECTOR/ c\IP_REPCOLLECTOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_REPAGGREGATOR/ c\IP_REPAGGREGATOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_GATEWAY/ c\IP_GATEWAY="'"$ip"'"' $conf
 
  echo -e "Wrote new docker IP addresses." | tee -a "$TMP_LOG_FILE"
}

#sp2 update4
reconfigureDockerNetwork(){
  conf="/etc/opt/novell/cis/config"
  subnet=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
  if [ $? -ne 0 ]; then
      echo -e "Unable to get cisnet subnet." | tee -a "$TMP_LOG_FILE"
      printandExit 1
  fi

  sub=${subnet/\/*/}

  if [ sub != "172.16.0.0" ]; then
    #stop services, remove docker network and recreate docker network
    stopServices
    if [ $? -ne 0 ]; then
      echo -e "Unable to stop cis services." | tee -a "$TMP_LOG_FILE"   
      printandExit 1         
    fi

    docker network rm cisnet 2>/dev/null
    if [ $? -ne 0 ]; then
      echo -e "Unable to remove docker network." | tee -a "$TMP_LOG_FILE"   
      printandExit 1         
    fi

    /bin/bash /opt/novell/cis/bin/activate_cis.sh

    writeDockerNwToConf
    if [ $? -ne 0 ]; then
      echo -e "Unable to write docker network to configuration file." | tee -a "$TMP_LOG_FILE"   
      printandExit 1         
    fi

  fi

}

add_dashboard(){
  config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

  if { [ "$config_Type" == "standalone" ] || [ "$config_Type" == "core" ]; };then
    conf='/etc/opt/novell/cis/config'
    repagg='REP_AGGREGATOR_NAME="cis-repaggregator"'
    dashboard_line="IP_DASHBOARD="
    dashboard='# Dashboard service Configuration\nDASHBOARD_NAME="oes-dashboard"\nDASHBOARD_THREADS="10"'

    isStringPresent "$dashboard_line" "$conf" 
    if [ "$?" == 0 ]; then
      #edit configuration file
      sed -i 's/'"$repagg"'/&\n'"$dashboard"'/' $conf

      #add to docker network
      gatewaystring=$(grep IP_GATEWAY "$conf")    
      dashboard=$(grep IP_GATEWAY "$conf"| cut -d'=' -f2 | xargs | cut -d'.' -f1-3 | xargs)
      dashboardip='IP_DASHBOARD="'"$dashboard"'.12"'
      sed -i 's/'"$gatewaystring"'/&\n'"$dashboardip"'/' $conf

      #load docker image
      docker load -i /usr/share/suse-docker-images/sles-oes-dashboard-*
      IMAGE_VERSION=`rpm -q --queryformat "%{VERSION}-%{RELEASE}" sles-oes-dashboard-docker-image`
      CUR_VERSION=`docker images microfocus/oes-dashboard | grep "microfocus/oes-dashboard" | awk -F ' ' '{print $2}'`
      docker tag microfocus/oes-dashboard:$CUR_VERSION microfocus/oes-dashboard:$IMAGE_VERSION

      systemctl enable oes-dashboard.service      

      echo -e "Updated docker network for dashboard. \n" | tee -a "$TMP_LOG_FILE"
    fi
  fi
}

backupCertificates(){
  echo -e "Backing up existing CIS certificates." | tee -a "$TMP_LOG_FILE"
  if ! { [ -e /etc/opt/novell/cis/backup/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/backup/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/backup/certs/rootCAs/SSCert.pem ]; }; then
    mv /etc/opt/novell/cis/certs/servercert.pem /etc/opt/novell/cis/backup/certs
    mv /etc/opt/novell/cis/certs/serverkey.pem /etc/opt/novell/cis/backup/certs
    mv /etc/opt/novell/cis/certs/rootCAs/SSCert.pem /etc/opt/novell/cis/backup/certs/rootCAs
  fi

  cp /etc/ssl/servercerts/servercert.pem /etc/opt/novell/cis/certs
  cp /etc/ssl/servercerts/serverkey.pem /etc/opt/novell/cis/certs
  cp /etc/opt/novell/certs/SSCert.pem /etc/opt/novell/cis/certs/rootCAs
  
  /opt/novell/cis/bin/appendcert -dir /etc/opt/novell/certs/ -leaf /etc/ssl/servercerts/servercert.pem -chain /etc/opt/novell/cis/certs/rootCAs/SSCert.pem

  ## Fix Kafka certificates
  echo -e "Importing certs in kafka key store." | tee -a "$TMP_LOG_FILE"
  KAFKA_ADV_ADDR=$(grep "MQ_ADVERTISED_ADDRESS" /etc/opt/novell/cis/kafka/config | cut -d'=' -f2- | xargs)
  if [[ $KAFKA_ADV_ADDR != "" ]]; then
    if ! { [ -e /etc/opt/novell/cis/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/certs/rootCAs/SSCert.pem ]; }; then
      echo "Server certificates not found"  | tee -a "$TMP_LOG_FILE"
      printandExit 1;
    fi

    config_file=/etc/opt/novell/cis/kafka/config
    password=$(head /dev/urandom | tr -dc A-Za-z0-9 | head -c 13 ; echo '')
    rm -f /etc/opt/novell/cis/kafka/certs/*

    openssl pkcs12 -export -in /etc/opt/novell/cis/certs/servercert.pem -inkey /etc/opt/novell/cis/certs/serverkey.pem -out /etc/opt/novell/cis/kafka/certs/server.p12 -name localhost -CAfile /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -caname rootca -password pass:"$password";  >> "$TMP_LOG_FILE" 2>&1
    if [ $? -ne 0 ]; then
      echo "Failed to import the server certificate, private key and CA certificate into PKCS12 keystore"  | tee -a "$TMP_LOG_FILE"
      printandExit 1;
    fi

    keytool -importkeystore -deststorepass "$password" -destkeypass "$password" -destkeystore /etc/opt/novell/cis/kafka/certs/server.keystore.jks -srckeystore /etc/opt/novell/cis/kafka/certs/server.p12 -srcstoretype PKCS12 -srcstorepass "$password" -alias localhost; >> "$TMP_LOG_FILE" 2>&1
    if [ $? -ne 0 ]; then
      echo "Failed to Covert the PKCS12 type keystore to JKS type keystore"  | tee -a "$TMP_LOG_FILE"
      printandExit 1;
    fi

    keytool -keystore /etc/opt/novell/cis/kafka/certs/server.truststore.jks -storepass "$password" -alias CARoot -import -file /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -noprompt >> "$TMP_LOG_FILE" 2>&1
    if [ $? -ne 0 ]; then
      echo "Failed to write MQ_KEYSTORE_PLAINTEXT_PASSWORD in kafka config file"  | tee -a "$TMP_LOG_FILE"
      printandExit 1;
    fi

    sed -i -e"s/^MQ_KEYSTORE_PLAINTEXT_PASSWORD=.*/MQ_KEYSTORE_PLAINTEXT_PASSWORD=\"$password\"/" "$config_file"
    sed -i -e"s/^MQ_KEY_PLAINTEXT_PASSWORD=.*/MQ_KEY_PLAINTEXT_PASSWORD=\"$password\"/" "$config_file"
    sed -i -e"s/^MQ_TRUSTSTORE_PLAINTEXT_PASSWORD=.*/MQ_TRUSTSTORE_PLAINTEXT_PASSWORD=\"$password\"/" "$config_file"
  fi
}

RSAKeyPair(){  
  if { [ -f /etc/opt/novell/cis/certs/epublickey.pem ] && [ -f /etc/opt/novell/cis/certs/eprivatekey.pem ]; };then
    echo -e "RSA key-pair already exists." | tee -a "$TMP_LOG_FILE"
    return 0
  else
    echo -e "No RSA key-pair found. Creating." | tee -a "$TMP_LOG_FILE"
    return 1           
  fi  
}

updateZookeeperWithKeys(){
  configType=$1
  prefix_path=$2
  ZK_URI=`grep "ZK_URI" /etc/opt/novell/cis/config | cut -d'=' -f2- | xargs`
  ## Generate RSA key pairs 
  if [ $configType == "standalone" ]; then
    /opt/novell/cis/bin/cismigrate --verify -zk_uri "$ZK_URI" -key_pair "cert" >> "$TMP_LOG_FILE" 2>&1
    if [[ $? -eq 0 ]]; then
      if { ! [ -f /etc/opt/novell/cis/certs/epublickey.pem ] && ! [ -f /etc/opt/novell/cis/certs/eprivatekey.pem ]; };then
        /opt/novell/cis/bin/cismigrate --generate >> "$TMP_LOG_FILE" 2>&1
        if [[ $? -ne 0 ]]; then
          echo -e "Failed to re-generate CIS encryption keys." >> "$TMP_LOG_FILE" 2>&1
          printandExit 1
        fi
      fi
    fi      
  else #core or cluster
    /opt/novell/cis/bin/cismigrate   --verify -zk_uri "$ZK_URI" -key_pair "cert" >> "$TMP_LOG_FILE" 2>&1
    if [[ $? -eq 0 ]]; then
      echo -e "Move to rsa keypair." >> "$TMP_LOG_FILE" 2>&1
      /opt/novell/cis/bin/cismigrate --standalone -zk_uri "$ZK_URI" -prefix_path "$prefix_path" >> "$TMP_LOG_FILE" 2>&1
      
      if [[ $? -eq 0 ]]; then 
        /opt/novell/cis/bin/cismigrate --verify -zk_uri "$ZK_URI" -key_pair "rsa" >> "$TMP_LOG_FILE" 2>&1
        if [[ $? -eq 1 ]]; then
          echo -e "Failed to verify CIS encryption keys." >> "$TMP_LOG_FILE" 2>&1
          printandExit 1
        fi
      elif [[ $? -eq 1 ]]; then
        echo -e "Failed to re-generate CIS encryption keys." >> "$TMP_LOG_FILE" 2>&1
        printandExit 1 
      fi   
    fi
  fi
}

updateZookeeper(){
  #check if zookeeper has to be updated
  ZK_URI=`grep "ZK_URI" /etc/opt/novell/cis/config | cut -d'=' -f2- | xargs`
  prefix_path=$1
  /opt/novell/cis/bin/cismigrate --update -zk_uri "$ZK_URI" -prefix_path "$prefix_path" >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Failed to update new values in zk." >> "$TMP_LOG_FILE" 2>&1
    printandExit 1
  fi
}

upgradeCisStandalone(){  
  prefix_path=$1
  # Stopping all CIS core services, If running
  declare -a cis_service_names=(
      "gateway" "mgmt" 
      "collector" "policy" 
      "repcollector" "aggregator" 
      "metadata" "auth" 
      "data" "repaggregator"
      "gatewayforscale" "dataatscale"
      "configuration"
  )
  for name in "${cis_service_names[@]}"
  do
      echo -e "Stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
      systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  # Stopping all CIS external dependencies, If running
  declare -a cis_ext_service_names=(
      "elastic" "kafka"
      "zk" "fluentbit"
  )
  for name in "${cis_ext_service_names[@]}"
  do
      echo -e "Stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
      systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  # Removing dangling container images
  docker stop cis-fluentbit >> "$TMP_LOG_FILE" 2>&1

  count=`docker ps -a -f "status=created" -f "status=exited" -f "status=dead" -q`;
  if [[ $count != "" ]]; then
      docker rm $(docker ps -a -f "status=created" -f "status=exited" -f "status=dead" -q) >> "$TMP_LOG_FILE" 2>&1
  fi

  echo -e "Updating CIS service docker images." | tee -a "$TMP_LOG_FILE"
  declare -a service_names=(
      "gateway" "mgmt"
      "collector" "policy"
      "repcollector" "aggregator"
      "metadata" "auth"
      "data" "repaggregator"
  )
  for name in "${service_names[@]}"
  do
      /opt/novell/cis/bin/cis_image_version_update.sh "$name" >> "$TMP_LOG_FILE" 2>&1
  done

  #Generate RSA key pair if not present
  
  # Performing CIS migration
  # 1. Creating RSA Public/Private Keys
  # 2. UnWrapping existing CIS encrypy key using certificates and ReWrapping them using RSA key pairs.
  backupCertificates
  
  systemctl daemon-reload

  systemctl enable oes-cis-fluentbit.service >> "$TMP_LOG_FILE" 2>&1

  systemctl is-active --quiet oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS zookeeper service."
      systemctl start oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS zookeeper service.\n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15
  systemctl is-active --quiet oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS kafka service. \n" | tee -a "$TMP_LOG_FILE"
      systemctl start oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS kafka service. \n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15
  systemctl is-active --quiet oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS elastic service."
      systemctl start oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
      systemctl is-active --quiet oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
      if [[ $? -ne 0 ]]; then
          echo -e "Failed to start CIS elastic service. \n" | tee -a "$TMP_LOG_FILE"
          printandExit 1
      fi
  fi
  sleep 15

  RSAKeyPair 
  keyPairCreated="$?"

  if [ "$keyPairCreated" == 1 ]; then
    updateZookeeperWithKeys "standalone" "$prefix_path"
  fi
  
  updateZookeeper "$prefix_path"
}

upgradeCisCore(){
  prefix_path=$1
  # Stopping all CIS core services, If running
  declare -a cis_service_names=(
      "gateway" "mgmt"
      "collector" "policy"
      "repcollector" "aggregator"
      "metadata" "auth"
      "data" "repaggregator"
      "gatewayforscale" "dataatscale"
      "configuration"
  )
  for name in "${cis_service_names[@]}"
  do
    echo -e "Stopping oes-cis-$name.service. \n" | tee -a "$TMP_LOG_FILE"
    systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  # Removing dangling container images
  docker stop cis-fluentbit >> "$TMP_LOG_FILE" 2>&1
  
  count=`docker ps -a -f "status=created" -f "status=exited" -f "status=dead" -q`;
  if [[ $count != "" ]]; then
      docker rm $(docker ps -a -f "status=created" -f "status=exited" -f "status=dead" -q) >> "$TMP_LOG_FILE" 2>&1
  fi

  echo -e "Updating CIS service docker images. \n" | tee -a "$TMP_LOG_FILE"
  declare -a service_names=(
      "gateway" "mgmt"
      "collector" "policy"
      "repcollector" "aggregator"
      "metadata" "auth"
      "data" "repaggregator"
  )
  for name in "${service_names[@]}"
  do
      /opt/novell/cis/bin/cis_image_version_update.sh "$name" >> "$TMP_LOG_FILE" 2>&1
  done

  systemctl daemon-reload

  systemctl enable oes-cis-fluentbit.service >> "$TMP_LOG_FILE" 2>&1

  # Performing CIS migration
  # 1. Creating RSA Public/Private Keys
  # 2. UnWrapping existing CIS encrypy key using certificates and ReWrapping them using RSA key pairs.
  
  RSAKeyPair 

  keyPairCreated="$?"
  if [ "$keyPairCreated" == 1 ]; then
    updateZookeeperWithKeys "core" "$prefix_path"
  fi

  updateZookeeper "$prefix_path"  
}

upgradeScale(){
    # Stopping all CIS core services, If running
    declare -a cis_service_names=(
        "gatewayforscale" "dataatscale"
        "configuration"
    )
    for name in "${cis_service_names[@]}"
    do
	    echo -e "Stopping oes-cis-$name.service. \n" | tee -a "$TMP_LOG_FILE"
        systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
    done

    ZK_URI=`grep "ZK_URI" /etc/opt/novell/cis-scale/config | cut -d'=' -f2- | xargs`
    CIS_SERVER=`grep "CIS_SERVER" /etc/opt/novell/cis-scale/config | cut -d'=' -f2- | xargs`
    /opt/novell/cis/bin/cismigrate --scale -zk_uri "$ZK_URI" -cis_server "$CIS_SERVER"  >> "$TMP_LOG_FILE" 2>&1
    if [[ $? -ne 0 ]]; then
        echo -e "Failed to register CIS scale server. \n" | tee -a "$TMP_LOG_FILE"
        printandExit 1
    fi
    conf='/etc/opt/novell/cis-scale/config'
    grep "^CIS_NET" $conf
    if [[ $? -ne 0 ]]; then
        echo -e "\nCIS_NET=\"cisnet\"" >> "$conf"
    fi
    sh /opt/novell/cis/bin/activate_cis_scale.sh  >> "$TMP_LOG_FILE" 2>&1        
}

updateMySql(){
  ## Performing MySQL re-config on upgrade if not already done    
  if [ -e /etc/my.cnf ]; then 
    isStringPresent "0.0.0.0" /etc/my.cnf
    if [ "$?" == 0 ]; then
      line='/bind-address/s/127.0.0.1/0.0.0.0/'
      sed -i -e "$line" /etc/my.cnf
      echo -e "Updated mysql conf file." | tee -a "$TMP_LOG_FILE"
    fi
  fi
}

updateConfiguration() {
  conf='/etc/opt/novell/cis/config'
  line='/etc/opt/novell/cis_prefixes/letc_cis/'

  # if line exists, we have to move to etc/opt
  isStringPresent "$line" "$conf"
  if [ "$?" == 0 ]; then
    sed -i '/^CERTS_PATH/ c\CERTS_PATH=\"/etc/opt/novell/cis/certs\"' "$conf"
    sed -i '/^CRED_PATH/ c\CRED_PATH=\"/etc/opt/novell/cis/.creds\"' "$conf"
    sed -i '/^CONFIG_PATH/ c\CONFIG_PATH=\"/etc/opt/novell/cis\"' "$conf"
    sed -i '/^CONFIGSERVICE_LOGPATH/ c\CONFIGSERVICE_LOGPATH=\"/var/opt/novell/log/cis/cisconfiguration.log\"' "$conf"
    sed -i '/^DB_CLIENT_CERT_PATH/ c\DB_CLIENT_CERT_PATH=\"/etc/opt/novell/cis/db/certs/\"' "$conf"
  fi

  es_conf='/etc/opt/novell/cis/es/config'
  isStringPresent "$line" "$es_conf"
  if [ "$?" == 0 ]; then
    sed -i '/^SERVER_CERTS_PATH/ c\SERVER_CERTS_PATH=\"/etc/opt/novell/cis/certs\"' "$es_conf"
    sed -i '/^ES_DATA_PATH/ c\ES_DATA_PATH=\"/etc/opt/novell/cis/es/data\"' "$es_conf"
  fi

  zk_conf='/etc/opt/novell/cis/zk/config'
  isStringPresent "$line" "$zk_conf"
  if [ "$?" == 0 ]; then
    sed -i '/^SERVER_CERTS_PATH/ c\SERVER_CERTS_PATH=\"/etc/opt/novell/cis/certs\"' "$zk_conf"
    sed -i '/^ZK_DATALOG_PATH/ c\ZK_DATALOG_PATH=\"/etc/opt/novell/cis/zk/datalog\"' "$zk_conf"
    sed -i '/^ZK_DATA_PATH/ c\ZK_DATA_PATH=\"/etc/opt/novell/cis/zk/data\"' "$zk_conf"
  fi

  kf_conf='/etc/opt/novell/cis/kafka/config'
  isStringPresent "$line" "$kf_conf"
  if [ "$?" == 0 ]; then
    sed -i '/^SERVER_CERTS_PATH/ c\SERVER_CERTS_PATH=\"/etc/opt/novell/cis/certs\"' "$kf_conf"
    sed -i '/^MQ_LOGS_PATH/ c\MQ_LOGS_PATH=\"/etc/opt/novell/cis/kafka/kafka-logs\"' "$kf_conf"
    sed -i '/^MQ_CERTS_PATH/ c\MQ_CERTS_PATH=\"/etc/opt/novell/cis/kafka/certs\"' "$kf_conf"
  fi

  sed -i '/path/ c\path /var/opt/novell/log/cis/microservices/cis.*.log' /etc/opt/novell/cis/fluentbit/fluentd.conf
  sed -i 's/cis_prefixes\/letc_cis/cis/g' /etc/opt/novell/cis/configurationStatus.json
  rm /etc/opt/novell/cis_prefixes/letc_cis
  rm /etc/opt/novell/cis_prefixes/lvar_logcis

  [[ ! -d /etc/opt/novell/cis/gateway ]] && cp -rp /etc/opt/novell/cis_local/gateway /etc/opt/novell/cis/
  [[ ! -f /etc/opt/novell/cis/cis3.yml ]] && cp -p /etc/opt/novell/cis_local/cis3.yml /etc/opt/novell/cis/cis3.yml
  [[ ! -f /etc/opt/novell/cis/cis5.yml ]] && cp -p /etc/opt/novell/cis_local/cis5.yml /etc/opt/novell/cis/cis5.yml
  [[ ! -f /etc/opt/novell/cis/cis7.yml ]] && cp -p /etc/opt/novell/cis_local/cis7.yml /etc/opt/novell/cis/cis7.yml
  [[ ! -f /etc/opt/novell/cis/status.json ]] && cp -p /etc/opt/novell/cis_local/status.json /etc/opt/novell/cis/status.json

}

# Entry Point
while [ "$1" != "" ]; do
  case $1 in
    --help )             usage
                         exit 0
                         ;;
    -h )                 usage
                         exit 0
                         ;;
    --version )          version
                         exit 0
                         ;;
    -v )                 version
                         exit 0
                         ;;
    * )                  usage
                         exit 1
                         ;;
    esac
                         shift
done

#check if links are not broken
if { [ -d /etc/opt/novell/cis_prefixes ] && [ ! -e /etc/opt/novell/cis_prefixes/letc_cis ]; }; then
  echo -e "NSS resource not mounted on the current node." | tee -a "$TMP_LOG_FILE"
  printandExit 1
else
  if { [ -L /etc/opt/novell/cis ] && [ ! -e /etc/opt/novell/cis ]; };then
    echo -e "NSS resource not mounted on the current node." | tee -a "$TMP_LOG_FILE"
    printandExit 1
  fi      
fi

prefix_path=""
current_version=""
is_Configured=""

#Todo prefix path should be non empty for sp2.0 or if any updates have happened
#media should be mounted else flag an error

if { [ -L /etc/opt/novell/cis_prefixes/letc_cis ] && [ -L /etc/opt/novell/cis_prefixes/lvar_logcis ]; };then
  prefix_path=`cat /etc/opt/novell/cis_prefixes/letc_cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'prefixPath'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
  is_Configured=`cat /etc/opt/novell/cis_prefixes/letc_cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'cisConfigSuccess'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
  current_version="SP2.0"
  echo -e "cis_prefixes link exists." | tee -a "$TMP_LOG_FILE"
else
  is_Configured=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'cisConfigSuccess'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
  prefix_path=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'prefixPath'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
  current_version=$(awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'patch'\042/){print $(i+1)}}}' /etc/opt/novell/cis/configurationStatus.json | tr -d '"' | xargs)
fi

echo -e "CIS prefix path is $prefix_path" | tee -a "$TMP_LOG_FILE"

if [ "$current_version" == "$SP3_0" ]; then
  echo -e "CIS is already upgraded to SP3. Exiting upgrade." | tee -a "$TMP_LOG_FILE"
  printandExit 0
fi

updateMySql

if [ ! -d /etc/opt/novell/cis_local ] ; then
  if { [ ! -L /etc/opt/novell/cis ] && [ "$current_version" != "SP2.0" ]; }; then
    echo -e "Creating symlinks to cis_local " | tee -a "$TMP_LOG_FILE"
    /bin/mv  /etc/opt/novell/cis /etc/opt/novell/cis_local      
    ln -fsn /etc/opt/novell/cis_local /etc/opt/novell/cis
  else 
    echo -e "Creating cis_local. No symlinks. " | tee -a "$TMP_LOG_FILE"
    /bin/cp -R /etc/opt/novell/cis /etc/opt/novell/cis_local 
  fi
fi
if [ ! -d /var/opt/novell/log/cis_local ] ; then
  if { [ ! -L /etc/opt/novell/cis ] && [ "$current_version" != "SP2.0" ]; }; then
    /bin/mv  /var/opt/novell/log/cis /var/opt/novell/log/cis_local
    ln -fsn /var/opt/novell/log/cis_local /var/opt/novell/log/cis
  else
    /bin/cp -R  /var/opt/novell/log/cis /var/opt/novell/log/cis_local
  fi
fi

if [ "$prefix_path" == "" ]; then 
  if [ "$is_Configured" == "false" ]; then
    echo -e "CIS is not yet configured." | tee -a "$TMP_LOG_FILE"  
    prepareSymLinks
    printandExit 0 
  else 
    #this is some old version less than sp1 update 9
    #we will move to /
    echo -e "CIS is configured. But prefix path is empty. Moving to /" | tee -a "$TMP_LOG_FILE"  
    prepareSymLinks       
    prefix_path="/"
  fi 
fi

# media path already present in sp2.0
if [ "$current_version" == "SP2.0" ]; then
  #if links are not broken earlier conf files refere to cis_prefixes we have to reset to /etc/cis
  updateConfiguration
  
  if [ "$prefix_path" == "/" ]; then
    echo -e "Creating default symlinks" | tee -a "$TMP_LOG_FILE"
    # Create default symlinks
    prepareSymLinks
  else
    echo -e "Creating symlinks to media " | tee -a "$TMP_LOG_FILE"
    # Create symlinks to media directory
    prepareSymLinks "$prefix_path"/etc/opt/novell/cis "$prefix_path"/var/opt/novell/log/cis  
  fi    
fi

#copy all new configuration to the attached cis now
if [ -d /etc/opt/novell/cis/gateway/conf/resolv.conf ]; then
  echo -e "Removed cis resolv directory." | tee -a "$TMP_LOG_FILE"
  rm -rf /etc/opt/novell/cis/gateway/conf/resolv.conf
fi
if [ -f /etc/opt/novell/cis/nginx.conf ]; then
  rm /etc/opt/novell/cis/nginx.conf
fi
/usr/bin/sh /opt/novell/cis/bin/copy_upgraded_files.sh "$prefix_path"

# Get configuration status and type.
config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

echo -e "Beginning CIS service upgrade." | tee -a "$TMP_LOG_FILE"
echo -e "CIS services configured in $config_Type mode" | tee -a "$TMP_LOG_FILE"
echo -e "Configuration path: $prefix_path" | tee -a "$TMP_LOG_FILE"

if { [ "$config_Type" == "standalone" ]; };then    
    upgradeCisStandalone "$prefix_path"    
    add_dashboard
fi

if { [ "$config_Type" == "core" ]; };then
    upgradeCisCore "$prefix_path"
    reconfigureDockerNetwork
    add_dashboard
fi

if { [ "$config_Type" == "cluster" ]; };then
    upgradeCisCore "$prefix_path"
    reconfigureDockerNetwork
    add_dashboard
fi

if { [ "$config_Type" == "scale" ]; };then
    upgradeScale
fi

updateConfigStatus "$prefix_path"
printandExit 0